require 'observer'

module PredicsisMlSdk
  class Job
    include Observable
    include Util

    attr_accessor :id

    def self.get(id, token = bearer_token)
      new(id, token).get
    end

    def self.all(token = bearer_token)
      data = Requests.get_jobs(token)
      data.each_with_object([]) do |e, a|
        a << new(e['id'], token).tap do |resource|
          resource.send(:define_attributes, e)
        end
      end
    end

    def initialize(id, token = bearer_token)
      @token = token
      @id = id
    end

    def get
      data = Requests.get_job(token, id)
      define_attributes(data)
      self
    end

    def delete
      Requests.delete_job(token, id)
      self
    end

    def next_job
      return nil unless next_job_id
      @next_job ||= Job.get(next_job_id, token)
    end

    def succeeded?
      status.eql? 'completed'
    end

    def failed?
      status.eql? 'failed'
    end

    def pending?
      %w(pending processing).include? status
    end

    def terminated?
      if failed?
        fail JobError.new(errors, id, action)
      elsif succeeded?
        next_job ? next_job.terminated? : true
      else
        false
      end
    end
  end
end
