module PredicsisMlSdk
  class Source
    include Util
    include JobStuff

    attr_accessor :id, :token

    def self.create(source_params = {}, data_store_params = {}, token = bearer_token)
      data = Requests.post_source(token, source_params, data_store: data_store_params)
      new(data['id'], token).tap do |source|
        source.define_attributes(data)
      end
    end

    def self.all(token = bearer_token)
      data = Requests.get_sources(token)
      data.each_with_object([]) do |e, a|
        a << new(e['id'], token).tap do |resource|
          resource.send(:define_attributes, e)
        end
      end
    end

    def self.get(id, token = bearer_token)
      new(id, token).get
    end

    def self.update(id, params = {}, token = bearer_token)
      new(id, token).update(params)
    end

    def self.delete(id, token = bearer_token)
      new(id, token).delete
    end

    def initialize(id, token = bearer_token)
      @token = token
      @id = id
    end

    def update(params = {})
      data = Requests.patch_source(token, id, params)
      define_attributes(data)
      self
    end

    def delete
      Requests.delete_source(token, id)
      self
    end

    def get
      data = Requests.get_source(token, id)
      define_attributes(data)
      self
    end
  end
end
