require 'predicsis_ml_sdk/upload_helper'
require 'ruby-progressbar'

module PredicsisMlSdk
  class Upload
    include Util
    include UploadHelper

    def self.start(file, token = bearer_token)
      Upload.initiate.tap do |u|
        u.upload(file)
      end
    end

    def self.initiate(token = bearer_token)
      data = Requests.initiate_upload(token)
      new(data['id'], data['path'], token).tap do |upload|
        upload.define_attributes(data)
      end
    end

    def self.get_part_url(id, path, part_number, token = bearer_token)
      new(id, path, token).get_part_url(part_number)
    end

    def self.complete(id, path, token = bearer_token)
      new(id, path, token).complete
    end

    def initialize(id, path, token = bearer_token)
      @id = id
      @path = path
      @token = token
    end

    def upload(file)
      path = split_into_files(file)
      nb_of_parts(file).times do |i|
        tmp_path = path + i.to_s.rjust(5, '0')
        sleep(1) until exists?(tmp_path, i, file)
        Requests.upload(get_part_url(i + 1), tmp_path, i + 1)
        progressbar.increment
      end
      Thread.new { FileUtils.rm_rf Dir.glob(path.gsub('part_', '')) }
      complete
    end

    def get_part_url(part_number)
      Requests.get_part_url(token, id, part_number, path)['part_url']
    end

    def complete
      Requests.complete_upload(token, id, path)
    end

    def to_h
      instance_variables.each_with_object({}) do |var, hash|
        hash[var.to_s.delete("@")] = instance_variable_get(var) unless var.to_s == '@token'
      end
    end

    def path
      @path ||= defined?(key) ? key : object
    end

    def progressbar
      @progressbar ||= ProgressBar.create(format: '|%b>%i| %p%% %t', starting_at: 0, :total => @nb_of_parts)
    end
  end
end
